# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .resource_list_params import ResourceListParams as ResourceListParams
from .resource_create_params import ResourceCreateParams as ResourceCreateParams
from .fixed_cost_resource_create_params import FixedCostResourceCreateParams as FixedCostResourceCreateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["KpiCreateParams"]


class KpiCreateParams(TypedDict, total=False):
    description: Required[str]

    goal: Required[float]

    kpi_type: Required[Literal["boolean", "number", "percentage", "likert5", "likert7", "likert10"]]

    name: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, TypedDict

__all__ = ["PropertyUpdateParams"]


class PropertyUpdateParams(TypedDict, total=False):
    category: Required[str]

    properties: Required[Dict[str, str]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .shared import (
    IngestUnits as IngestUnits,
    XproxyError as XproxyError,
    XproxyResult as XproxyResult,
    PropertiesResponse as PropertiesResponse,
    PayICommonModelsBudgetManagementCostDetailsBase as PayICommonModelsBudgetManagementCostDetailsBase,
    PayICommonModelsBudgetManagementCreateLimitBase as PayICommonModelsBudgetManagementCreateLimitBase,
)
from .cost_data import CostData as CostData
from .cost_details import CostDetails as CostDetails
from .requests_data import RequestsData as RequestsData
from .limit_response import LimitResponse as LimitResponse
from .request_result import RequestResult as RequestResult
from .ingest_response import IngestResponse as IngestResponse
from .total_cost_data import TotalCostData as TotalCostData
from .default_response import DefaultResponse as DefaultResponse
from .category_response import CategoryResponse as CategoryResponse
from .limit_list_params import LimitListParams as LimitListParams
from .ingest_bulk_params import IngestBulkParams as IngestBulkParams
from .ingest_event_param import IngestEventParam as IngestEventParam
from .limit_reset_params import LimitResetParams as LimitResetParams
from .ingest_units_params import IngestUnitsParams as IngestUnitsParams
from .limit_create_params import LimitCreateParams as LimitCreateParams
from .limit_list_response import LimitListResponse as LimitListResponse
from .limit_update_params import LimitUpdateParams as LimitUpdateParams
from .bulk_ingest_response import BulkIngestResponse as BulkIngestResponse
from .category_list_params import CategoryListParams as CategoryListParams
from .limit_history_response import LimitHistoryResponse as LimitHistoryResponse
from .category_delete_response import CategoryDeleteResponse as CategoryDeleteResponse
from .category_resource_response import CategoryResourceResponse as CategoryResourceResponse
from .use_case_instance_response import UseCaseInstanceResponse as UseCaseInstanceResponse
from .category_list_resources_params import CategoryListResourcesParams as CategoryListResourcesParams
from .category_delete_resource_response import CategoryDeleteResourceResponse as CategoryDeleteResourceResponse
from .pay_i_common_models_api_router_header_info_param import (
    PayICommonModelsAPIRouterHeaderInfoParam as PayICommonModelsAPIRouterHeaderInfoParam,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from payi import Payi, AsyncPayi
from tests.utils import assert_matches_type
from payi.pagination import SyncCursorPage, AsyncCursorPage
from payi.types.use_cases import KpiListResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestKpis:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_update(self, client: Payi) -> None:
        kpi = client.use_cases.kpis.update(
            kpi_name="kpi_name",
            use_case_id="use_case_id",
        )
        assert kpi is None

    @parametrize
    def test_method_update_with_all_params(self, client: Payi) -> None:
        kpi = client.use_cases.kpis.update(
            kpi_name="kpi_name",
            use_case_id="use_case_id",
            score=True,
        )
        assert kpi is None

    @parametrize
    def test_raw_response_update(self, client: Payi) -> None:
        response = client.use_cases.kpis.with_raw_response.update(
            kpi_name="kpi_name",
            use_case_id="use_case_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        kpi = response.parse()
        assert kpi is None

    @parametrize
    def test_streaming_response_update(self, client: Payi) -> None:
        with client.use_cases.kpis.with_streaming_response.update(
            kpi_name="kpi_name",
            use_case_id="use_case_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            kpi = response.parse()
            assert kpi is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Payi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `use_case_id` but received ''"):
            client.use_cases.kpis.with_raw_response.update(
                kpi_name="kpi_name",
                use_case_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `kpi_name` but received ''"):
            client.use_cases.kpis.with_raw_response.update(
                kpi_name="",
                use_case_id="use_case_id",
            )

    @parametrize
    def test_method_list(self, client: Payi) -> None:
        kpi = client.use_cases.kpis.list(
            use_case_id="use_case_id",
        )
        assert_matches_type(SyncCursorPage[KpiListResponse], kpi, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Payi) -> None:
        kpi = client.use_cases.kpis.list(
            use_case_id="use_case_id",
            cursor="cursor",
            kpi_name="kpi_name",
            limit=0,
            sort_ascending=True,
        )
        assert_matches_type(SyncCursorPage[KpiListResponse], kpi, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Payi) -> None:
        response = client.use_cases.kpis.with_raw_response.list(
            use_case_id="use_case_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        kpi = response.parse()
        assert_matches_type(SyncCursorPage[KpiListResponse], kpi, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Payi) -> None:
        with client.use_cases.kpis.with_streaming_response.list(
            use_case_id="use_case_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            kpi = response.parse()
            assert_matches_type(SyncCursorPage[KpiListResponse], kpi, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: Payi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `use_case_id` but received ''"):
            client.use_cases.kpis.with_raw_response.list(
                use_case_id="",
            )


class TestAsyncKpis:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_update(self, async_client: AsyncPayi) -> None:
        kpi = await async_client.use_cases.kpis.update(
            kpi_name="kpi_name",
            use_case_id="use_case_id",
        )
        assert kpi is None

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncPayi) -> None:
        kpi = await async_client.use_cases.kpis.update(
            kpi_name="kpi_name",
            use_case_id="use_case_id",
            score=True,
        )
        assert kpi is None

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncPayi) -> None:
        response = await async_client.use_cases.kpis.with_raw_response.update(
            kpi_name="kpi_name",
            use_case_id="use_case_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        kpi = await response.parse()
        assert kpi is None

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncPayi) -> None:
        async with async_client.use_cases.kpis.with_streaming_response.update(
            kpi_name="kpi_name",
            use_case_id="use_case_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            kpi = await response.parse()
            assert kpi is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncPayi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `use_case_id` but received ''"):
            await async_client.use_cases.kpis.with_raw_response.update(
                kpi_name="kpi_name",
                use_case_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `kpi_name` but received ''"):
            await async_client.use_cases.kpis.with_raw_response.update(
                kpi_name="",
                use_case_id="use_case_id",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncPayi) -> None:
        kpi = await async_client.use_cases.kpis.list(
            use_case_id="use_case_id",
        )
        assert_matches_type(AsyncCursorPage[KpiListResponse], kpi, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncPayi) -> None:
        kpi = await async_client.use_cases.kpis.list(
            use_case_id="use_case_id",
            cursor="cursor",
            kpi_name="kpi_name",
            limit=0,
            sort_ascending=True,
        )
        assert_matches_type(AsyncCursorPage[KpiListResponse], kpi, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncPayi) -> None:
        response = await async_client.use_cases.kpis.with_raw_response.list(
            use_case_id="use_case_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        kpi = await response.parse()
        assert_matches_type(AsyncCursorPage[KpiListResponse], kpi, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncPayi) -> None:
        async with async_client.use_cases.kpis.with_streaming_response.list(
            use_case_id="use_case_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            kpi = await response.parse()
            assert_matches_type(AsyncCursorPage[KpiListResponse], kpi, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncPayi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `use_case_id` but received ''"):
            await async_client.use_cases.kpis.with_raw_response.list(
                use_case_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .total_cost_data import TotalCostData

__all__ = ["LimitHistoryResponse", "LimitHistory"]


class LimitHistory(BaseModel):
    limit_id: Optional[str] = None

    limit_name: Optional[str] = None

    totals: TotalCostData

    limit_reset_timestamp: Optional[datetime] = None

    limit_tags: Optional[List[str]] = None

    limit_type: Optional[Literal["block", "allow"]] = None

    max: Optional[float] = None


class LimitHistoryResponse(BaseModel):
    limit_history: LimitHistory

    request_id: str

    message: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .result import (
    ResultResource,
    AsyncResultResource,
    ResultResourceWithRawResponse,
    AsyncResultResourceWithRawResponse,
    ResultResourceWithStreamingResponse,
    AsyncResultResourceWithStreamingResponse,
)
from .properties import (
    PropertiesResource,
    AsyncPropertiesResource,
    PropertiesResourceWithRawResponse,
    AsyncPropertiesResourceWithRawResponse,
    PropertiesResourceWithStreamingResponse,
    AsyncPropertiesResourceWithStreamingResponse,
)
from .request_id import (
    RequestIDResource,
    AsyncRequestIDResource,
    RequestIDResourceWithRawResponse,
    AsyncRequestIDResourceWithRawResponse,
    RequestIDResourceWithStreamingResponse,
    AsyncRequestIDResourceWithStreamingResponse,
)

__all__ = [
    "ResultResource",
    "AsyncResultResource",
    "ResultResourceWithRawResponse",
    "AsyncResultResourceWithRawResponse",
    "ResultResourceWithStreamingResponse",
    "AsyncResultResourceWithStreamingResponse",
    "PropertiesResource",
    "AsyncPropertiesResource",
    "PropertiesResourceWithRawResponse",
    "AsyncPropertiesResourceWithRawResponse",
    "PropertiesResourceWithStreamingResponse",
    "AsyncPropertiesResourceWithStreamingResponse",
    "RequestIDResource",
    "AsyncRequestIDResource",
    "RequestIDResourceWithRawResponse",
    "AsyncRequestIDResourceWithRawResponse",
    "RequestIDResourceWithStreamingResponse",
    "AsyncRequestIDResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["KpiUpdateParams"]


class KpiUpdateParams(TypedDict, total=False):
    use_case_name: Required[str]

    description: Optional[str]

    goal: Optional[float]
